@@ -2,7 +2,7 @@
   mxBeeBase -- BeeBase C extension
 
   Copyright (c) 1998-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-  Copyright (c) 2000-2007, eGenix.com Software GmbH; mailto:info@egenix.com
+  Copyright (c) 2000-2008, eGenix.com Software GmbH; mailto:info@egenix.com
   See the documentation for further copyright information or contact
   the author (mailto:mal@lemburg.com).
 
@@ -27,7 +27,7 @@
 #include "mxBeeBase.h"
 
 /* Version number: Major.Minor.Patchlevel */
-#define MXBEEBASE_VERSION "3.0.0"
+#define MXBEEBASE_VERSION "3.1.0"
 
 /* Define these to have the module use free lists (saves malloc calls) */
 /*#define MXBEEINDEX_FREELIST*/
@@ -40,7 +40,7 @@ static char *Module_docstring =
  MXBEEBASE_MODULE" -- BeeBase objects and functions. Version "MXBEEBASE_VERSION"\n\n"
 
  "Copyright (c) 1998-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com\n"
- "Copyright (c) 2000-2007, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
+ "Copyright (c) 2000-2008, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
  "                 All Rights Reserved\n\n"
  "See the documentation for further information on copyrights,\n"
  "or contact the author."
@@ -336,7 +336,7 @@ void mxBeeIndex_Free(mxBeeIndexObject *beeindex)
 
 /* --- Key management routines --- */
 
-/* Use Python strings as keys */
+/* Use Python strings as keys (these may not contain embedded NULs) */
 static
 void *mxBeeIndex_KeyFromString(mxBeeIndexObject *beeindex,
 			       PyObject *key)
@@ -365,9 +365,46 @@ PyObject *mxBeeIndex_StringFromKey(mxBeeIndexObject *beeindex,
 }
 
 static
-int mxBeeIndex_CompareStrings(const void *key1, const void *key2) 
+int mxBeeIndex_CompareStrings(size_t keysize, 
+			      const void *key1, 
+			      const void *key2) 
 {
-    return strcmp((char*)key1,(char*)key2);
+    return strcmp((char*)key1, (char*)key2);
+}
+
+/* Use fixed length Python strings as keys (these may contain embedded
+   NULs) */
+static
+void *mxBeeIndex_KeyFromFixedLengthString(mxBeeIndexObject *beeindex,
+					  PyObject *key)
+{
+    Py_Assert(PyString_Check(key),
+	      PyExc_TypeError,
+	      "keys must be strings");
+    Py_AssertWithArg((int)PyString_GET_SIZE(key) == beeindex->info.keySize - 1,
+		     PyExc_TypeError,
+		     "keys must have fixed length %i",beeindex->info.keySize - 1);
+
+    return (void*)PyString_AS_STRING(key);
+    
+ onError:
+    return NULL;
+}
+
+static
+PyObject *mxBeeIndex_FixedLengthStringFromKey(mxBeeIndexObject *beeindex,
+					      void *key)
+{
+    return PyString_FromStringAndSize((char*)key, 
+				      (Py_ssize_t)(beeindex->info.keySize - 1));
+}
+
+static
+int mxBeeIndex_CompareFixedLengthStrings(size_t keysize, 
+					 const void *key1, 
+					 const void *key2) 
+{
+    return memcmp((char*)key1, (char*)key2, keysize);
 }
 
 /* Use Python integer as keys */
@@ -392,7 +429,9 @@ PyObject *mxBeeIndex_IntegerFromKey(mxBeeIndexObject *beeindex,
 }
 
 static
-int mxBeeIndex_CompareLongs(const void *key1, const void *key2) 
+int mxBeeIndex_CompareLongs(size_t keysize, 
+			    const void *key1, 
+			    const void *key2) 
 {
     unsigned long a = *(unsigned long *)key1;
     unsigned long b = *(unsigned long *)key2;
@@ -421,7 +460,9 @@ PyObject *mxBeeIndex_FloatFromKey(mxBeeIndexObject *beeindex,
 }
 
 static
-int mxBeeIndex_CompareDoubles(const void *key1, const void *key2) 
+int mxBeeIndex_CompareDoubles(size_t keysize, 
+			      const void *key1, 
+			      const void *key2) 
 {
     double a = *(double *)key1;
     double b = *(double *)key2;
@@ -1663,6 +1704,32 @@ Py_C_Function_WithKeywords(
     return NULL;
 }
 
+Py_C_Function_WithKeywords(
+    mxBeeIndex_BeeFixedLengthStringIndex,
+    "BeeFixedLengthStringIndex(filename,keysize,dupkeys=0,filemode=0,sectorsize=256)\n\n"
+    )
+{
+    char *filename;
+    int keysize;
+    int sectorsize = 256;
+    int dupkeys = 0;
+    int filemode = 0;
+
+    Py_KeywordsGet5Args("si|iii",
+			filename,keysize,dupkeys,filemode,sectorsize);
+
+    return (PyObject *)mxBeeIndex_New(filename,
+				      filemode,
+				      keysize + 1,
+				      sectorsize,
+				      mxBeeIndex_CompareFixedLengthStrings,
+				      mxBeeIndex_FixedLengthStringFromKey,
+				      mxBeeIndex_KeyFromFixedLengthString,
+				      dupkeys);
+ onError:
+    return NULL;
+}
+
 Py_C_Function_WithKeywords(
     mxBeeIndex_BeeIntegerIndex,
     "BeeIntegerIndex(filename,dupkeys=0,filemode=0,sectorsize=256)\n\n"
@@ -1720,9 +1787,14 @@ Py_C_Function_WithKeywords(
 static 
 PyMethodDef Module_methods[] =
 {   
-    Py_MethodWithKeywordsListEntry("BeeStringIndex",mxBeeIndex_BeeStringIndex),
-    Py_MethodWithKeywordsListEntry("BeeIntegerIndex",mxBeeIndex_BeeIntegerIndex),
-    Py_MethodWithKeywordsListEntry("BeeFloatIndex",mxBeeIndex_BeeFloatIndex),
+    Py_MethodWithKeywordsListEntry("BeeStringIndex",
+				   mxBeeIndex_BeeStringIndex),
+    Py_MethodWithKeywordsListEntry("BeeFixedLengthStringIndex",
+				   mxBeeIndex_BeeFixedLengthStringIndex),
+    Py_MethodWithKeywordsListEntry("BeeIntegerIndex",
+				   mxBeeIndex_BeeIntegerIndex),
+    Py_MethodWithKeywordsListEntry("BeeFloatIndex",
+				   mxBeeIndex_BeeFloatIndex),
     {NULL,NULL} /* end of list */
 };
 
