@@ -1,7 +1,7 @@
 """ Python part of the low-level DateTime[Delta] type implementation.
 
     Copyright (c) 1998-2001, Marc-Andre Lemburg; mailto:mal@lemburg.com
-    Copyright (c) 2000-2007, eGenix.com Software GmbH; mailto:info@egenix.com
+    Copyright (c) 2000-2008, eGenix.com Software GmbH; mailto:info@egenix.com
     See the documentation for further information on copyrights,
     or contact the author. All Rights Reserved.
 """
@@ -1019,7 +1019,7 @@ def add_century(year,
         If adding the current century to the given year gives a year
         within the range current_year-70...current_year+30 [both
         inclusive], then the current century is added. Otherwise the
-        century (current + 1 or - 1) producing the least difference is
+        century (current + 1 or - 1) producing the smallest difference is
         chosen.
 
     """
