@@ -3,7 +3,7 @@
 """ RegExp - tag a string using regexps (Version 0.1)
     
     Copyright (c) 2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-    Copyright (c) 2000-2007, eGenix.com Software GmbH; mailto:info@egenix.com
+    Copyright (c) 2000-2008, eGenix.com Software GmbH; mailto:info@egenix.com
     See the documentation for further information on copyrights,
     or contact the author. All Rights Reserved.
 """
@@ -13,7 +13,7 @@ from mx.TextTools import *
 
 # special matching function
 def match_regexp(text,position,len_text,regexpr):
-    position = position + max(0,regexpr.match(text,position))
+    position = position + max(0,regexpr.match(text,position).end())
     return position
 
 # create a table in the sense re_1\|re_2\|...\|re_N where
@@ -29,9 +29,9 @@ def or_regexps(regexps):
 if __name__ == '__main__':
 
     # create some simple regexps
-    import regex
+    import re
     regexps = [ 'spam*', 'ham*', 'eggs' ]
-    regexps = map(regex.compile,regexps)
+    regexps = map(re.compile, regexps)
     table = or_regexps(regexps)
 
     text = raw_input('type some words: ')
