@@ -1,7 +1,7 @@
 """ mxTextTools - A tools package for fast text processing.
 
     Copyright (c) 2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-    Copyright (c) 2000-2007, eGenix.com Software GmbH; mailto:info@egenix.com
+    Copyright (c) 2000-2008, eGenix.com Software GmbH; mailto:info@egenix.com
     See the documentation for further information on copyrights,
     or contact the author. All Rights Reserved.
 """
@@ -28,11 +28,13 @@ from Constants.Sets import *
 #
 def format_entry(table,i,
 
-                 TupleType=types.TupleType):
+                 TupleType=types.TupleType,StringTypes=types.StringTypes):
 
     """ Returns a pp-formatted tag table entry as string 
     """
     e = table[i]
+    if type(e) in StringTypes:
+        return '%-15.15s : %s' % ('', e)
     jne = 0
     je = 1
     t,c,m = e[:3]
@@ -48,7 +50,11 @@ def format_entry(table,i,
         m = repr(m)
         if len(m) > 17:
             m = m[:17]+'...'
-    return '%-15.15s : %-30s : jne=%+i : je=%+i' % \
+    if type(jne) not in StringTypes:
+        jne = '%+i' % jne
+    if type(je) not in StringTypes:
+        je = '%+i' % je
+    return '%-15.15s : %-30s : jne=%s : je=%s' % \
            (repr(t),'%-.15s : %s'%(c,m),jne,je)
 
 def format_table(table,i=-1):
