@@ -2,7 +2,7 @@
   mxTools -- Misc. utilities for Python
 
   Copyright (c) 1997-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-  Copyright (c) 2000-2007, eGenix.com Software GmbH; mailto:info@egenix.com
+  Copyright (c) 2000-2008, eGenix.com Software GmbH; mailto:info@egenix.com
   See the documentation for further copyright information or contact
   the author (mailto:mal@lemburg.com).
 
@@ -34,7 +34,12 @@
 /* Needed for mxTools_cur_frame() */
 #include "frameobject.h"
 
-#define MXTOOLS_VERSION "3.0.0"
+/* Needed for mxTools_dlopen() */
+#ifdef HAVE_DLOPEN
+# include <dlfcn.h>
+#endif
+
+#define MXTOOLS_VERSION "3.1.0"
 
 /* To open up the playground for new ideas... */
 /*#define INCLUDE_FUNSTUFF*/
@@ -49,7 +54,7 @@ static char *Module_docstring =
  MXTOOLS_MODULE" -- A tool collection. Version "MXTOOLS_VERSION"\n\n"
 
  "Copyright (c) 1997-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com\n"
- "Copyright (c) 2000-2007, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
+ "Copyright (c) 2000-2008, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
  "                 All Rights Reserved\n\n"
  "See the documentation for further information on copyrights,\n"
  "or contact the author."
@@ -121,7 +126,7 @@ PyObject *insexc(PyObject *moddict,
     return v;
 }
 
-#if 0
+#ifdef HAVE_DLOPEN
 /* Helper for adding integer constants. Check for errors with
    PyErr_Occurred() */
 static 
@@ -2155,6 +2160,92 @@ Py_C_Function( mxTools_dictscan,
     return NULL;
 }
 
+#ifdef HAVE_PY_GETARGCARGV
+
+/* This API is available in all Python version since at least 2.1, but
+   not made public via the Python.h header file. */
+extern void Py_GetArgcArgv(int *argc, char ***argv);
+
+Py_C_Function( mxTools_setproctitle,
+	       "setproctitle(title)\n\n"
+	       "Set the process title."
+	       )
+{
+    char *title;
+    Py_ssize_t titlelen;
+    int argc;
+    char **argv;
+    int i, argv0_len;
+    char *argv0;
+    
+    Py_Get2Args("s#", title, titlelen);
+
+    Py_GetArgcArgv(&argc, &argv);
+
+    /* Check how much space we can use to save the title. We have to
+       be careful here, since the argv array items may not be all
+       pointing to the same memory area. */
+    argv0 = argv[0];
+    argv0_len = strlen(argv0) + 1;
+    for (i = 1; i < argc; i++) {
+	if (argv[i] == argv0 + argv0_len)
+	    argv0_len += strlen(argv[i]) + 1;
+        else
+	    break;
+    }
+
+    /* Don't count the last 0-termination byte */
+    argv0_len -= 1;
+
+    /* Copy the new title to the static buffer for the process title
+       and 0-terminate it. */
+    titlelen = min(titlelen, argv0_len);
+    memcpy(argv0, title, titlelen);
+    for (i = titlelen; i < argv0_len; i++)
+	argv0[i] = '\0';
+
+    /* Truncate the argv list and set all remaining entries to NULL */
+    for (i = 1; i < argc; i++)
+	argv[i] = 0;
+    
+    Py_ReturnNone();
+
+ onError:
+    return NULL;
+}
+
+#endif
+
+#ifdef HAVE_DLOPEN
+
+Py_C_Function( mxTools_dlopen,
+	       "dlopen(libname, mode)\n\n"
+	       "Load the shared lib libname using the flags given in mode.\n"
+	       "mode defaults to Python's standard dlopenflags."
+	       )
+{
+    char *libname;
+    int mode = PyThreadState_GET()->interp->dlopenflags;
+    void *handle = 0;
+    
+    Py_Get2Args("s|i", libname, mode);
+
+    handle = dlopen(libname, mode);
+    if (handle == NULL) {
+	/* Get error information */
+	const char *error = dlerror();
+	if (error == NULL)
+	    error = "unknown dlopen() error";
+	Py_Error(PyExc_OSError, error);
+    }
+    return PyLong_FromVoidPtr(handle);
+
+ onError:
+    return NULL;
+}
+
+#endif
+
 /* Helper for mxTools_verscmp */
 
 static
@@ -2509,51 +2600,57 @@ static PyTypeObject mxNotGiven_Type = {
 static 
 PyMethodDef Module_methods[] =
 {   
-    Py_MethodListEntry("trange",mxTools_trange),
-    Py_MethodListEntrySingleArg("trange_len",mxTools_indices),
-    Py_MethodListEntrySingleArg("indices",mxTools_indices),
-    Py_MethodListEntrySingleArg("range_len",mxTools_range_len),
-    Py_MethodListEntry("irange",mxTools_irange),
-    Py_MethodListEntry("ifilter",mxTools_ifilter),
-    Py_MethodListEntry("get",mxTools_get),
-    Py_MethodListEntry("extract",mxTools_extract),
-    Py_MethodListEntry("iremove",mxTools_iremove),
-    Py_MethodListEntry("mget",mxTools_extract),
-    Py_MethodListEntry("mgetattr",mxTools_findattr),
-    Py_MethodListEntry("findattr",mxTools_findattr),
-    Py_MethodListEntry("attrlist",mxTools_attrlist),
-    Py_MethodListEntry("mapply",mxTools_mapply),
-    Py_MethodListEntry("method_mapply",mxTools_method_mapply),
-    Py_MethodListEntry("count",mxTools_count),
-    Py_MethodListEntry("exists",mxTools_exists),
-    Py_MethodListEntry("forall",mxTools_forall),
-    Py_MethodListEntry("index",mxTools_index),
-    Py_MethodListEntry("napply",mxTools_napply),
-    Py_MethodListEntrySingleArg("sizeof",mxTools_sizeof),
-    Py_MethodListEntrySingleArg("dict",mxTools_dict),
-    Py_MethodListEntrySingleArg("invdict",mxTools_invdict),
-    Py_MethodListEntry("setdict",mxTools_setdict),
-    Py_MethodListEntrySingleArg("reverse",mxTools_reverse),
-    Py_MethodListEntrySingleArg("tuples",mxTools_tuples),
-    Py_MethodListEntrySingleArg("lists",mxTools_lists),
-    Py_MethodListEntry("acquire",mxTools_acquire),
-    Py_MethodListEntry("verbosity",mxTools_verbosity),
-    Py_MethodListEntry("optimization",mxTools_optimization),
-    Py_MethodListEntry("interactive",mxTools_interactive),
-    Py_MethodListEntry("debugging",mxTools_debugging),
-    Py_MethodListEntry("cur_frame",mxTools_cur_frame),
-    Py_MethodListEntry("truth",mxTools_truth),
-    Py_MethodListEntry("sign",mxTools_sign),
-    Py_MethodListEntry("makeref",mxTools_makeref),
-    Py_MethodListEntry("dictscan",mxTools_dictscan),
-    Py_MethodListEntry("verscmp",mxTools_verscmp),
+    Py_MethodListEntry("trange", mxTools_trange),
+    Py_MethodListEntrySingleArg("trange_len", mxTools_indices),
+    Py_MethodListEntrySingleArg("indices", mxTools_indices),
+    Py_MethodListEntrySingleArg("range_len", mxTools_range_len),
+    Py_MethodListEntry("irange", mxTools_irange),
+    Py_MethodListEntry("ifilter", mxTools_ifilter),
+    Py_MethodListEntry("get", mxTools_get),
+    Py_MethodListEntry("extract", mxTools_extract),
+    Py_MethodListEntry("iremove", mxTools_iremove),
+    Py_MethodListEntry("mget", mxTools_extract),
+    Py_MethodListEntry("mgetattr", mxTools_findattr),
+    Py_MethodListEntry("findattr", mxTools_findattr),
+    Py_MethodListEntry("attrlist", mxTools_attrlist),
+    Py_MethodListEntry("mapply", mxTools_mapply),
+    Py_MethodListEntry("method_mapply", mxTools_method_mapply),
+    Py_MethodListEntry("count", mxTools_count),
+    Py_MethodListEntry("exists", mxTools_exists),
+    Py_MethodListEntry("forall", mxTools_forall),
+    Py_MethodListEntry("index", mxTools_index),
+    Py_MethodListEntry("napply", mxTools_napply),
+    Py_MethodListEntrySingleArg("sizeof", mxTools_sizeof),
+    Py_MethodListEntrySingleArg("dict", mxTools_dict),
+    Py_MethodListEntrySingleArg("invdict", mxTools_invdict),
+    Py_MethodListEntry("setdict", mxTools_setdict),
+    Py_MethodListEntrySingleArg("reverse", mxTools_reverse),
+    Py_MethodListEntrySingleArg("tuples", mxTools_tuples),
+    Py_MethodListEntrySingleArg("lists", mxTools_lists),
+    Py_MethodListEntry("acquire", mxTools_acquire),
+    Py_MethodListEntry("verbosity", mxTools_verbosity),
+    Py_MethodListEntry("optimization", mxTools_optimization),
+    Py_MethodListEntry("interactive", mxTools_interactive),
+    Py_MethodListEntry("debugging", mxTools_debugging),
+    Py_MethodListEntry("cur_frame", mxTools_cur_frame),
+    Py_MethodListEntry("truth", mxTools_truth),
+    Py_MethodListEntry("sign", mxTools_sign),
+    Py_MethodListEntry("makeref", mxTools_makeref),
+    Py_MethodListEntry("dictscan", mxTools_dictscan),
+    Py_MethodListEntry("verscmp", mxTools_verscmp),
+#ifdef HAVE_PY_GETARGCARGV
+    Py_MethodListEntry("setproctitle", mxTools_setproctitle),
+#endif
+#ifdef HAVE_DLOPEN
+    Py_MethodListEntry("dlopen",  mxTools_dlopen),
+#endif
 #ifdef INCLUDE_FUNSTUFF
-    Py_MethodListEntrySingleArg("malloc",mxTools_malloc),
-    Py_MethodListEntry("caching_eval",mxTools_caching_eval),
+    Py_MethodListEntrySingleArg("malloc", mxTools_malloc),
+    Py_MethodListEntry("caching_eval", mxTools_caching_eval),
 #endif
 #ifdef MXTOOLS_ENABLE_SAFECALL
-    Py_MethodListEntry("safecall",mxTools_safecall),
-    Py_MethodListEntry("segfault",mxTools_segfault),
+    Py_MethodListEntry("safecall", mxTools_safecall),
+    Py_MethodListEntry("segfault", mxTools_segfault),
 #endif
     {NULL,NULL} /* end of list */
 };
@@ -2607,11 +2704,34 @@ MX_EXPORT(void)
 
     /* Add some constants to the module's dict */
     moddict = PyModule_GetDict(module);
-    insstr(moddict,"__version__",MXTOOLS_VERSION);
+    insstr(moddict, "__version__", MXTOOLS_VERSION);
     PyDict_SetItemString(moddict,
 			 "NotGiven",
 			 mxNotGiven);
 
+    /* dlopen() mode flags */
+#ifdef RTLD_LAZY
+    insint(moddict, "RTLD_LAZY", RTLD_LAZY);
+#endif
+#ifdef RTLD_NOW
+    insint(moddict, "RTLD_NOW", RTLD_NOW);
+#endif
+#ifdef RTLD_NOLOAD
+    insint(moddict, "RTLD_NOLOAD", RTLD_NOLOAD);
+#endif
+#ifdef RTLD_DEEPBIND
+    insint(moddict, "RTLD_DEEPBIND", RTLD_DEEPBIND);
+#endif
+#ifdef RTLD_GLOBAL
+    insint(moddict, "RTLD_GLOBAL", RTLD_GLOBAL);
+#endif
+#ifdef RTLD_LOCAL
+    insint(moddict, "RTLD_LOCAL", RTLD_LOCAL);
+#endif
+#ifdef RTLD_NODELETE
+    insint(moddict, "RTLD_NODELETE", RTLD_NODELETE);
+#endif
+
     /* Errors */
     if (!(mxTools_Error = insexc(moddict, "Error", NULL)))
 	goto onError;
